
/*
 *  Project     [tie]
 *  Author      TriX
 *  Description TI92 linker for the parallel & serial cable
 *  Module      lowlevel driver for ti92 link interface
 */

#ifndef DJGPP
#error  Currently only for DJGPP availabel
#endif

#include "ti92.h"
#include <dos.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>

int SetPort (int Port)
{
   char            DeviceName[32];

   if (Port & COM_PORT_1)
   {
      sprintf (DeviceName, "/dev/ttyS%d", Port - COM_PORT_1);
      if ((Device = fopen (DeviceName, "r+")) == NULL)
      {
         printf ("Error opening communication port\n");
         exit (1);
      } /* if */
   } else {
      /* LPT Port */
      Device = NULL;
      dosmemget (0x408 + (Port - LPT_PORT_1) * 2, 2, &LPT_Port);
   } /* if */
   return 0;
} /* SetPort */

int ClosePort ()
{
   if (Device != NULL) fclose (Device);
   return 0;
} /* ClosePort */

#define  lptin  LPT_Port + 1
#define  lptout LPT_Port
/* Wait for the TI92 Acknowledge (LPT Routine) */
int Wait (int Code, int Until)
{
   struct time t;
   long   timeout = 1 << 30;

   while ((inportb (lptin) & Code) != Until && --timeout > 0);
   if (timeout == 0)
      return 0;
   else
      return 1;
} /* Wait */


int SendByte (unsigned char byte)
{
   if (Device != NULL)
      fputc (byte, Device);
   else {
      /* LPT putbyte routine */
     int n, b = byte;

     printf (" S %02X", byte);
     for (n = 0; n < 8; n++)
     {
        outportb (lptout, 0x3);
        if ((b & (1 << n)) == 0)
        {
           /* Bit not set */
           outportb (lptout, 0x1);
           /* Acknowledge from the TI92 */
           if (!Wait (0x20, 0)) return 0;
           outportb (lptout, 0x3);
           if (!Wait (0x20, 0x20)) return 0;
        } else {
           /* Bit set */
           outportb (lptout, 0x2);
           /* Acknowledge from the TI92 */
           if (!Wait (0x10, 0)) return 0;
           outportb (lptout, 0x3);
           if (!Wait (0x10, 0x10)) return 0;
        } /* if */
     } /* for */
   } /* if */
   return 1;
} /* SendByte */

int GetByte (unsigned char *byte)
{
   if (Device != NULL)
      *byte = fgetc (Device);
   else {
      /* LPT putbyte routine */
      int n, x = 0, i;
      long timeout;

      for (n = 0; n < 8; n++)
      {
         i = 0x30;
         timeout = 1 << 30;
         while (i == 0x30 && timeout > 0)
         {
            i = inportb (lptin) & 0x30;
            timeout--;
         } /* while */
         if (i == 0x30 || timeout == 0) return 0;
         if (i == 0x10)
         {
            /* Bit = 1 */
            x += (1 << n);
            outportb (lptout, 0x1);
            if (!Wait (0x20, 0x20)) return 0;
         } else {
            /* Bit = 0 */
            outportb (lptout, 0x2);
            if (!Wait (0x10, 0x10)) return 0;
         } /* if */
         outportb (lptout, 0x3);
      } /* for */
      printf (" G %02X", x);
      *byte = (unsigned char) x;
   } /* if */
   return 1;
} /* GetByte */

int SendHeader (TIHeader tih)
{
   if (!SendByte (0x9)) return 0;
   SendByte (tih.Type & 0xff);
   SendByte ((tih.Stuff >> 8) & 0xff);
   SendByte (tih.Stuff & 0xff);
   return 1;
} /* SendHeader */

int SendHInts (int type, int stuff)
{
   if (!SendByte (0x9)) return 0;
   SendByte (type & 0xff);
   SendByte ((stuff >> 8) & 0xff);
   SendByte (stuff & 0xff);
   return 1;
}

int GetChar ()
{
   unsigned char tmp;

   GetByte (&tmp);

   return tmp;
}


int GetWord ()
{
   unsigned char tmp[2];

   GetByte (&tmp[0]);
   GetByte (&tmp[1]);

   return (tmp[1] << 8) + (tmp[0]);   /* Conversion to the right format */
}


long GetDWord ()
{
   unsigned char tmp[4];

   GetByte (&tmp[0]);
   GetByte (&tmp[1]);
   GetByte (&tmp[2]);
   GetByte (&tmp[3]);

   return (tmp[3] << 24) + (tmp[2] << 16) + (tmp[1] << 8) + (tmp[0]);
}


int GetHeader (TIHeader *tih)
{
   unsigned char tmp;
   int k;

   if (!GetByte (&tmp)) return 0;
   k = tmp;
   if (k != 0x89) return 0;

   tih->Type = GetChar ();
   tih->Stuff = GetWord ();

   return 1;
} /* GetHeader */

int GetBody (char *savewhere, int *size)
{
   char  *myptr, tmp;
   int   i;

   TIHeader tih;

   if (!GetHeader (&tih))  return 0;
   if (tih.Type != 0x15)   return 0;

   GetChar (); GetChar (); GetChar (); GetChar ();

   myptr = savewhere;
   if (size != NULL) *size = tih.Stuff - 4;

   for (i = 0; i < tih.Stuff - 4; i++)
   {
      GetByte (myptr);
      myptr++;
   } /* for */

   GetByte (&tmp); GetByte (&tmp);          /* Ignore checksum */

   return 1;
} /* GetBody */

int SendVarHeader (VarHeader vh)
{
   int HSize = 6 + vh.Namelength, chksum, i;

   if (!SendHInts (0x06, ((HSize & 0xff) << 8) + ((HSize >> 8) & 0xff)))
      return 0;
   chksum = 0;

   SendByte ((vh.VariableSize >> 0)  & 0xff);
   SendByte ((vh.VariableSize >> 8)  & 0xff);
   SendByte ((vh.VariableSize >> 16) & 0xff);
   SendByte ((vh.VariableSize >> 24) & 0xff);

   chksum += (vh.VariableSize >> 0)  & 0xff;
   chksum += (vh.VariableSize >> 8)  & 0xff;
   chksum += (vh.VariableSize >> 16) & 0xff;
   chksum += (vh.VariableSize >> 24) & 0xff;

   SendByte (vh.VariableType);
   SendByte (vh.Namelength);

   for (i = 0; i < vh.Namelength; i++)
   {
      SendByte (vh.VariableName[i]);
      chksum += vh.VariableName[i];
   }

   chksum += (vh.VariableType & 0xff);
   chksum += (vh.Namelength   & 0xff);

   /* Send chechsum */
   SendByte ((chksum >> 0) & 0xff);
   SendByte ((chksum >> 8) & 0xff);

   return 1;
}

int GetVarHeader (VarHeader *Savein)
{
   int i;

   Savein->VariableSize = GetDWord ();
   Savein->VariableType = GetChar ();
   Savein->Namelength   = GetChar ();

   for (i = 0; i < Savein->Namelength; i++)
      Savein->VariableName[i] = GetChar ();

   Savein->VariableName[i] = '\0';

   Savein->Checksum = GetWord ();

   return 1;
}

